@@ -30,9 +30,9 @@ class Register:
 
 
 class InverterEquipmentRegister(Enum):
-    Model = Register(30000, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
-    SN = Register(30015, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
-    PN = Register(30025, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Model = Register(30000, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
+    SN = Register(30015, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
+    PN = Register(30025, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     ModelID = Register(30070, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     NumberOfPVStrings = Register(30071, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     NumberOfMPPTrackers = Register(30072, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
@@ -41,12 +41,12 @@ class InverterEquipmentRegister(Enum):
     MaximumApparentPower = Register(30077, 2, datatypes.DataType.UINT32_BE, 1000, "kVA", AccessType.RO, None)
     MaximumReactivePowerFedToTheGrid = Register(30079, 2, datatypes.DataType.INT32_BE, 1000, "kvar", AccessType.RO, None)
     MaximumReactivePowerAbsorbedFromTheGrid = Register(30081, 2, datatypes.DataType.INT32_BE, 1000, "kvar", AccessType.RO, None)
-    State1 = Register(32000, 1, datatypes.DataType.BITFIELD16, 1, None, AccessType.RO, None)
-    State2 = Register(32002, 1, datatypes.DataType.BITFIELD16, 1, None, AccessType.RO, None)
-    State3 = Register(32003, 2, datatypes.DataType.BITFIELD32, 1, None, AccessType.RO, None)
-    Alarm1 = Register(32008, 1, datatypes.DataType.BITFIELD16, 1, None, AccessType.RO, None)
-    Alarm2 = Register(32009, 1, datatypes.DataType.BITFIELD16, 1, None, AccessType.RO, None)
-    Alarm3 = Register(32010, 1, datatypes.DataType.BITFIELD16, 1, None, AccessType.RO, None)
+    State1 = Register(32000, 1, datatypes.DataType.BITFIELD16, None, None, AccessType.RO, None)
+    State2 = Register(32002, 1, datatypes.DataType.BITFIELD16, None, None, AccessType.RO, None)
+    State3 = Register(32003, 2, datatypes.DataType.BITFIELD32, None, None, AccessType.RO, None)
+    Alarm1 = Register(32008, 1, datatypes.DataType.BITFIELD16, None, None, AccessType.RO, None)
+    Alarm2 = Register(32009, 1, datatypes.DataType.BITFIELD16, None, None, AccessType.RO, None)
+    Alarm3 = Register(32010, 1, datatypes.DataType.BITFIELD16, None, None, AccessType.RO, None)
     PV1Voltage = Register(32016, 1, datatypes.DataType.INT16_BE, 10, "V", AccessType.RO, None)
     PV1Current = Register(32017, 1, datatypes.DataType.INT16_BE, 100, "A", AccessType.RO, None)
     PV2Voltage = Register(32018, 1, datatypes.DataType.INT16_BE, 10, "V", AccessType.RO, None)
@@ -98,9 +98,9 @@ class InverterEquipmentRegister(Enum):
     ActivePowerPercentageDerating = Register(40125, 1, datatypes.DataType.UINT16_BE, 10, "%", AccessType.RW, None)
     FixedActivePowerDeratedInW = Register(40126, 2, datatypes.DataType.UINT32_BE, 1, "W", AccessType.RW, None)
     ReactivePowerCompensationAtNight = Register(40129, 2, datatypes.DataType.INT32_BE, 1000, "kvar", AccessType.RW, None)
-    CosPhiPPnCharacteristicCurve = Register(40133, 21, datatypes.DataType.MULTIDATA, 1, None, AccessType.RW, None)
-    QUCharacteristicCurve = Register(40154, 21, datatypes.DataType.MULTIDATA, 1, None, AccessType.RW, None)
-    PFUCharacteristicCurve = Register(40175, 21, datatypes.DataType.MULTIDATA, 1, None, AccessType.RW, None)
+    CosPhiPPnCharacteristicCurve = Register(40133, 21, datatypes.DataType.MULTIDATA, None, None, AccessType.RW, None)
+    QUCharacteristicCurve = Register(40154, 21, datatypes.DataType.MULTIDATA, None, None, AccessType.RW, None)
+    PFUCharacteristicCurve = Register(40175, 21, datatypes.DataType.MULTIDATA, None, None, AccessType.RW, None)
     ReactivePowerAdjustmentTime = Register(40196, 1, datatypes.DataType.UINT16_BE, 1, "s", AccessType.RW, None)
     QUPowerPercentageToExitScheduling = Register(40198, 1, datatypes.DataType.UINT16_BE, 1, "%", AccessType.RW, None)
     # Startup = Register(40200, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.WO, None) # disabled because not readable (AccessType.WO)
@@ -128,7 +128,7 @@ class BatteryEquipmentRegister(Enum):
     TotalDischarge = Register(37782, 2, datatypes.DataType.INT32_BE, 100, "kWh", AccessType.RO, None)
     CurrentDayChargeCapacity = Register(37784, 2, datatypes.DataType.INT32_BE, 100, "kWh", AccessType.RO, None)
     CurrentDayDischargeCapacity = Register(37786, 2, datatypes.DataType.INT32_BE, 100, "kWh", AccessType.RO, None)
-    TimeOfUseElectricityPricePeriods = Register(47028, 41, datatypes.DataType.MULTIDATA, 1, None, AccessType.RW, None)
+    TimeOfUseElectricityPricePeriods = Register(47028, 41, datatypes.DataType.MULTIDATA, None, None, AccessType.RW, None)
     MaximumChargingPower = Register(47075, 2, datatypes.DataType.INT32_BE, 1, "W", AccessType.RW, None)
     MaximumDischargingPower = Register(47077, 2, datatypes.DataType.INT32_BE, 1, "W", AccessType.RW, None)
     ChargingCutoffCapacity = Register(47081, 1, datatypes.DataType.UINT16_BE, 10, "%", AccessType.RW, None)
@@ -137,13 +137,13 @@ class BatteryEquipmentRegister(Enum):
     ChargeFromGridFunction = Register(47087, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, mappings.ChargeFromGridFunction)
     GridChargeCutoffSOC = Register(47088, 1, datatypes.DataType.UINT16_BE, 10, "%", AccessType.RW, None)
     # ForcibleChargeDischarge = Register(47100, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.WO, mappings.ForcibleChargeDischarge) # disabled because not readable (AccessType.WO)
-    FixedChargingAndDischargingPeriods = Register(47200, 41, datatypes.DataType.MULTIDATA, 1, None, AccessType.RW, None)
+    FixedChargingAndDischargingPeriods = Register(47200, 41, datatypes.DataType.MULTIDATA, None, None, AccessType.RW, None)
     PowerOfChargeFromGrid = Register(47242, 2, datatypes.DataType.INT32_BE, 100, "kW", AccessType.RW, None)
     MaximumPowerOfChargeFromGrid = Register(47244, 2, datatypes.DataType.INT32_BE, 100, "kW", AccessType.RW, None)
     ForcibleChargeDischargeSettingMode = Register(47246, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, mappings.ForcibleChargeDischargeSettingMode)
     ForcibleChargePower = Register(47247, 2, datatypes.DataType.INT32_BE, 100, "kW", AccessType.RW, None)
     ForcibleDischargePower = Register(47249, 2, datatypes.DataType.INT32_BE, 100, "kW", AccessType.RW, None)
-    TimeOfUseChargingAndDischargingPeriods = Register(47255, 43, datatypes.DataType.MULTIDATA, 1, None, AccessType.RW, None)
+    TimeOfUseChargingAndDischargingPeriods = Register(47255, 43, datatypes.DataType.MULTIDATA, None, None, AccessType.RW, None)
     ExcessPVEnergyUseInTOU = Register(47299, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, mappings.ExcessPVEnergyUseInTOU)
     MaximumChargeFromGridPower = Register(47590, 2, datatypes.DataType.INT32_BE, 100, "kW", AccessType.RW, None)
     SwitchToOffGrid = Register(47604, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, mappings.SwitchToOffGrid)
@@ -151,11 +151,11 @@ class BatteryEquipmentRegister(Enum):
 
     # Unit 1
     Unit1ProductModel = Register(47000, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, mappings.ProductModel)
-    Unit1SN = Register(37052, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1SN = Register(37052, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1No = Register(47107, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit1SoftwareVersion = Register(37814, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
-    Unit1DCDCVersion = Register(37026, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
-    Unit1BMSVersion = Register(37036, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1SoftwareVersion = Register(37814, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
+    Unit1DCDCVersion = Register(37026, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
+    Unit1BMSVersion = Register(37036, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1RunningStatus = Register(37000, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, mappings.RunningStatus)
     Unit1WorkingMode = Register(37006, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, mappings.WorkingMode)
     Unit1BusVoltage = Register(37003, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
@@ -174,9 +174,9 @@ class BatteryEquipmentRegister(Enum):
 
     # Unit 2
     Unit2ProductModel = Register(47089, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, mappings.ProductModel)
-    Unit2SN = Register(37700, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2SN = Register(37700, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2No = Register(47108, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit2SoftwareVersion = Register(37799, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2SoftwareVersion = Register(37799, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2RunningStatus = Register(37741, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, mappings.RunningStatus)
     Unit2BusVoltage = Register(37750, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit2BusCurrent = Register(37751, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
@@ -189,9 +189,9 @@ class BatteryEquipmentRegister(Enum):
     Unit2BatteryTemperature = Register(37752, 1, datatypes.DataType.INT16_BE, 10, "°C", AccessType.RO, None)
 
     # Unit 1 BatteryPack 1
-    Unit1BatteryPack1SN = Register(38200, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1BatteryPack1SN = Register(38200, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1BatteryPack1No = Register(47750, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit1BatteryPack1FirmwareVersion = Register(38210, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1BatteryPack1FirmwareVersion = Register(38210, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1BatteryPack1WorkingStatus = Register(38228, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     Unit1BatteryPack1Voltage = Register(38235, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit1BatteryPack1Current = Register(38236, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
@@ -203,9 +203,9 @@ class BatteryEquipmentRegister(Enum):
     Unit1BatteryPack1MaximumTemperature = Register(38452, 1, datatypes.DataType.INT16_BE, 10, "°C", AccessType.RO, None)
 
     # Unit 1 BatteryPack 2
-    Unit1BatteryPack2SN = Register(38242, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1BatteryPack2SN = Register(38242, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1BatteryPack2No = Register(47751, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit1BatteryPack2FirmwareVersion = Register(38252, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1BatteryPack2FirmwareVersion = Register(38252, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1BatteryPack2WorkingStatus = Register(38270, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     Unit1BatteryPack2Voltage = Register(38277, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit1BatteryPack2Current = Register(38278, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
@@ -217,9 +217,9 @@ class BatteryEquipmentRegister(Enum):
     Unit1BatteryPack2MaximumTemperature = Register(38454, 1, datatypes.DataType.INT16_BE, 10, "°C", AccessType.RO, None)
 
     # Unit 1 BatteryPack 3
-    Unit1BatteryPack3SN = Register(38284, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1BatteryPack3SN = Register(38284, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1BatteryPack3No = Register(47752, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit1BatteryPack3FirmwareVersion = Register(38294, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit1BatteryPack3FirmwareVersion = Register(38294, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit1BatteryPack3WorkingStatus = Register(38312, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     Unit1BatteryPack3Voltage = Register(38319, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit1BatteryPack3Current = Register(38320, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
@@ -231,9 +231,9 @@ class BatteryEquipmentRegister(Enum):
     Unit1BatteryPack3MaximumTemperature = Register(38456, 1, datatypes.DataType.INT16_BE, 10, "°C", AccessType.RO, None)
 
     # Unit 2 BatteryPack 1
-    Unit2BatteryPack1SN = Register(38326, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2BatteryPack1SN = Register(38326, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2BatteryPack1No = Register(47753, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit2BatteryPack1FirmwareVersion = Register(38336, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2BatteryPack1FirmwareVersion = Register(38336, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2BatteryPack1WorkingStatus = Register(38354, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     Unit2BatteryPack1Voltage = Register(38361, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit2BatteryPack1Current = Register(38362, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
@@ -245,9 +245,9 @@ class BatteryEquipmentRegister(Enum):
     Unit2BatteryPack1MaximumTemperature = Register(38458, 1, datatypes.DataType.INT16_BE, 10, "°C", AccessType.RO, None)
 
     # Unit 2 BatteryPack 2
-    Unit2BatteryPack2SN = Register(38368, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2BatteryPack2SN = Register(38368, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2BatteryPack2No = Register(47754, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit2BatteryPack2FirmwareVersion = Register(38378, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2BatteryPack2FirmwareVersion = Register(38378, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2BatteryPack2WorkingStatus = Register(38396, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     Unit2BatteryPack2Voltage = Register(38403, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit2BatteryPack2Current = Register(38404, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
@@ -259,9 +259,9 @@ class BatteryEquipmentRegister(Enum):
     Unit2BatteryPack2MaximumTemperature = Register(38460, 1, datatypes.DataType.INT16_BE, 10, "°C", AccessType.RO, None)
 
     # Unit 2 BatteryPack 3
-    Unit2BatteryPack3SN = Register(38410, 10, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2BatteryPack3SN = Register(38410, 10, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2BatteryPack3No = Register(47755, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RW, None)
-    Unit2BatteryPack3FirmwareVersion = Register(38420, 15, datatypes.DataType.STRING, 1, None, AccessType.RO, None)
+    Unit2BatteryPack3FirmwareVersion = Register(38420, 15, datatypes.DataType.STRING, None, None, AccessType.RO, None)
     Unit2BatteryPack3WorkingStatus = Register(38438, 1, datatypes.DataType.UINT16_BE, 1, None, AccessType.RO, None)
     Unit2BatteryPack3Voltage = Register(38445, 1, datatypes.DataType.UINT16_BE, 10, "V", AccessType.RO, None)
     Unit2BatteryPack3Current = Register(38446, 1, datatypes.DataType.INT16_BE, 10, "A", AccessType.RO, None)
